/******************************************************************************
 * Author: Mario Roessler
 * Date:   2008/12/01
 * Description: puts characters from UART to a 4x27 LCD Display
 ******************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "lcd_1.h"
#include "uart.h"

#define F_CPU 16000000UL  // Clock speed = 16MHz
#define XTAL 16000000

#define delay(us)  _delayFourCycles( ( ( 1*(XTAL/4000) )*us)/1000 )

#define DISP_LINE_LENGTH    27
#define UART_BAUD_RATE      4800 


/***************************************************
 * Hello World
 ***************************************************/
void hello_world(void)
{
   // Hello
   uart_puts_P("\n\rGPS2LCD v0.8a\n\r");
   lcd_puts_P(1,"GPS2LCD v0.8a\n");
   lcd_puts_P(2,"by Mario Roessler\n");
}


/***************************************************
 * gps_findgpgga() - search for GPGGA header
 * just searching for 3 random characters followed
 * by "GA" and returns 1 of it was found or
 * 0 if not found
 ***************************************************/
int gps_findgpgga(void)
{
   char buffers;
   uint8_t skip = 3;
   /* skip first 3 characters */
   while(skip) {
     buffers = uart_getc();
     if (buffers && UART_NO_DATA) {
	   skip--;
	   //lcd_puts_P(2,"*");
     }
   }
   
   while (1==1) {
    buffers = uart_getc();
    if (buffers && UART_NO_DATA) {
      if (buffers == 'G') {
	    //lcd_puts_P(2,"G");
	    while(1==1) {
		  buffers = uart_getc();
          if (buffers && UART_NO_DATA) {
		    if (buffers == 'A') {
			  //lcd_puts_P(2,"A");
			  return 1;
            } else return 0;
          } 
        }
      } else return 0;
    }
   }
   //lcd_puts_P(2,"-");
   return 0;
}

/***************************************************
 * skip_dataset(skip), skip is one searches for the next ','
 ***************************************************/
void skip_dataset(uint8_t skip)
{
   char buffers;
   
   /* skip datasets */
   while(skip) {
     buffers = uart_getc();
     if (buffers && UART_NO_DATA) {
	   if (buffers == ',') {
	     skip--;
	     //lcd_puts_P(2,"*");
       }
     }
   }
}

/***************************************************
 * read_data(*buffer, number_of_characters)
 ***************************************************/
void read_data(char *buffer, uint8_t num)
{
  char bufferc;
  strncpy(buffer,"\0",1); //clear buffer before reading  

  while(num) {
     bufferc = uart_getc();
     if (bufferc && UART_NO_DATA) {
	   strncat(buffer,&bufferc,1);
	   num--;
     }
  }
}

/***************************************************
 * gps_getdata() - fill GPS data into string buffers
 * data is separated by ',' - search for it
 * afterwards read:
 * time (6 characters)
 * Position n/s (9 characters)
 * North or South (1 character)
 * Position e/w (10 characters)
 * East or West (1 character)
 * number of satellites (2 characters)
 * 
 ***************************************************/
void gps_getdata(void)
{
  char bufs[11];

  
  //lcd_clrscr(1);
  //lcd_clrscr(2);  

  skip_dataset(1); //search for start of next dataset
  read_data(bufs,6);
  lcd_gotoxy(1,0,1);
  lcd_puts_P(1,"Time: ");
  lcd_puts(1,bufs);
  
  skip_dataset(1);
  read_data(bufs,9);
  lcd_gotoxy(2,0,0);
  lcd_puts_P(2," ");
  lcd_puts(2,bufs);
  lcd_puts_P(2," ");

  skip_dataset(1);
  read_data(bufs,1);
  lcd_puts(2,bufs);

  skip_dataset(1);
  read_data(bufs,10);
  lcd_gotoxy(2,0,1);
  lcd_puts(2,bufs);
  lcd_puts_P(2," ");

  skip_dataset(1);
  read_data(bufs,1);
  lcd_puts(2,bufs);

  skip_dataset(2);
  read_data(bufs,2);
  lcd_puts_P(1,"    Sat: ");
  lcd_puts(1,bufs);

  skip_dataset(2);
  read_data(bufs,5);
  lcd_gotoxy(2,16,0);
  lcd_puts_P(2,"hNN: ");
  lcd_puts(2,bufs);
  lcd_puts_P(2,"m");

}

/***************************************************
 * gps_printraw() - print gps data to display
 * line 0 = N/S, line 1 = E/W, line 3 = time, line 4 = satellites
 ***************************************************/
void gps_printraw(void)
{

}


/***************************************************
 * Main Program
 ***************************************************/
int main (void)
{

   char buffer;
   char strbuffer[10];
   int loop;

   strcpy(strbuffer,"-\0");

   /* Initialize UART */
   uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );
   sei();  // enable Interrupts

   // Initialize Display
   lcd_init(1,LCD_DISP_ON);
   lcd_init(2,LCD_DISP_ON);
   lcd_clrscr(1);
   lcd_clrscr(2);

   DDRD |= (1<<PD5);    //set PD5 to output direction
   PORTD |= (1<<PD5);   //switch LED at PD5 on
   hello_world();
   /* wait for a while */

   loop=1000;
   while(loop)
   {
      delay(0xFF);
	  delay(0xFF);
	  delay(0xFF);
	  loop--;
   }
   PORTD &= ~(1<<PD5);  //switch LED off

   /* clear display */
   lcd_clrscr(1);
   lcd_clrscr(2);

   /* Main Loop */
   while(1==1)
   {
      //lcd_puts(2,strbuffer);
      /* Search loop for $GPRMC packet */
      /* Search for start of a line = $ character */
      buffer = uart_getc();
      if (buffer && UART_NO_DATA)
      {
        if (buffer == '$') {
		   //lcd_puts_P(2,"+"); //debug: a $ sign was found
	       if (gps_findgpgga()) {
 		      //lcd_clrscr(1);
			  lcd_gotoxy(1,0,0);
		      lcd_puts_P(1,"GPGGA packet count (");
		      loop++;
		 	  itoa(loop,strbuffer,10);
			  lcd_puts(1,strbuffer);
			  lcd_puts_P(1,")");
		      gps_getdata();
		   }
         //gps_printraw();
        }
      }
   
   }
   lcd_puts_P(2,"Assert!");
}
